import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Random;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingWorker;

public class ProgressBarDemo extends JPanel implements ActionListener, PropertyChangeListener {

	private static final long	serialVersionUID	= 1L;
	private JProgressBar		progressBar;
	private JTextArea			taskOutput;
	private static Task			task;
	private JFrame				frame;
	private JButton				closeButton;

	class Task extends SwingWorker<Void, Void> {

		// Main task. Executed in background thread.
		@Override
		public Void doInBackground() {
			final Random random = new Random();
			int progress = 0;
			// Initialize the progress property.
			this.setProgress(0);

			while (progress < 100) {
				// Sleep for up to one second.
				try {
					Thread.sleep(random.nextInt(1000));
				} catch (final InterruptedException ignore) {
				}

				// Make random progress.
				progress += random.nextInt(10);
				this.setProgress(Math.min(progress, 100));
			}

			return null;
		}

		// Executed in event dispatching thread
		@Override
		public void done() {
			Toolkit.getDefaultToolkit().beep();
			ProgressBarDemo.this.setCursor(null); // turn off the wait cursor
			ProgressBarDemo.this.taskOutput.append("Done!\n");
			ProgressBarDemo.this.closeButton.setEnabled(true);
		}
	}

	public ProgressBarDemo() {
		super(new BorderLayout());

		final Dimension progressBarDimension = new Dimension(450, 20), closeButtonDimension = new Dimension(100, 20);

		// Create the demo's UI.
		this.progressBar = new JProgressBar(0, 100);
		this.progressBar.setValue(0);
		this.progressBar.setStringPainted(true);
		this.progressBar.setPreferredSize(progressBarDimension);

		this.closeButton = new JButton("Close");
		this.closeButton.setPreferredSize(closeButtonDimension);
		this.closeButton.setActionCommand("close");
		this.closeButton.addActionListener(this);
		this.closeButton.setEnabled(false);

		this.taskOutput = new JTextArea(5, 20);
		this.taskOutput.setMargin(new Insets(5, 5, 5, 5));
		this.taskOutput.setEditable(false);

		final JPanel panel = new JPanel();
		panel.add(this.progressBar);

		this.add(panel, BorderLayout.PAGE_START);
		this.add(new JScrollPane(this.taskOutput), BorderLayout.CENTER);
		this.add(this.closeButton, BorderLayout.PAGE_END);
		this.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
	}

	/**
	 * Invoked when task's progress property changes.
	 */
	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
		if ("progress" == evt.getPropertyName()) {
			final int progress = (Integer) evt.getNewValue();
			this.progressBar.setValue(progress);
			this.taskOutput.append(String.format("Completed %d%% of task.\n", ProgressBarDemo.task.getProgress()));
		}
	}

	/**
	 * Create the GUI and show it. As with all GUI code, this must run on the event-dispatching thread.
	 */
	private void createAndShowGUI() {
		// Create and set up the window.
		this.frame = new JFrame("ProgressBarDemo");
		this.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		final Dimension dimension = new Dimension(500, 300);
		this.setSize(dimension);
		this.setPreferredSize(dimension);

		// Create and set up the content pane.
		final JComponent newContentPane = this;
		newContentPane.setOpaque(true); // content panes must be opaque
		this.frame.setContentPane(newContentPane);

		// Display the window.
		this.frame.pack();
		this.frame.setVisible(true);

		ProgressBarDemo.task = new Task();
		ProgressBarDemo.task.addPropertyChangeListener(this);
		ProgressBarDemo.task.execute();

		do {
			try {
				Thread.sleep(200);
			} catch (final InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} while (!task.isDone());
	}

	public void perform() {
		System.out.println("Start");

		// javax.swing.SwingUtilities.invokeLater(new Runnable() {
		final Thread monitorThread = new Thread(new Runnable() {
			@Override
			public void run() {
				ProgressBarDemo.this.createAndShowGUI();
			}
		});

		monitorThread.start();

		do {
			try {
				Thread.sleep(200);
			} catch (final InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} while (monitorThread.isAlive());

		System.out.println("The End");
	}

	public static void main(final String[] args) {
		final ProgressBarDemo progressBarDemo = new ProgressBarDemo();
		progressBarDemo.perform();
	}

	@Override
	public void actionPerformed(final ActionEvent paramActionEvent) {
		if (this.frame != null) {
			this.frame.dispose();
		}
	}
}
